# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2017 E-MIPS (http://www.e-mips.com.ar)
#    Copyright (c) 2017 Eynes (http://www.eynes.com.ar)
#    All Rights Reserved. See AUTHORS for details.
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from datetime import date
from openerp.tests import common
from openerp import netsvc
from openerp.osv.osv import except_osv
from functools import partial

class CommonSuitcaseFunc():

    def __init__(self, cr, uid, registry):
        self.cr = cr
        self.uid = uid
        self.registry = registry

        self.suitcase_model = self.registry('stock.suitcase')
        self.wiz_add_materials_model = self.registry('suitcase.add.materials.wizard')
        self.material_line_model = self.registry('suitcase.add.materials.line')
        self.stock_picking = self.registry('stock.picking')
        self.partial_picking = self.registry('stock.partial.picking')

    def _create_suitcase(self, name, employee):
        cr, uid = self.cr, self.uid

        suitcase_id = self.suitcase_model.create(cr, uid, dict(
            name=name, employee_id=employee,
        ))

        return self.suitcase_model.browse(cr, uid, suitcase_id)

    def _add_materials_to_suitcase(self, suitcase, product_id, location_id, qty):
        mode = 'in'
        return self._transfer_materials(suitcase, product_id, location_id, qty, mode)

    def _rem_materials_to_suitcase(self, suitcase, product_id, location_id, qty):
        mode = 'out'
        return self._transfer_materials(suitcase, product_id, location_id, qty, mode)

    def _transfer_materials(self, suitcase, product_id, location_id, qty, mode):
        cr, uid = self.cr, self.uid

        context = {'active_id': suitcase.id,
                   'active_ids': [suitcase.id],
                   'sense': mode == 'in' and 'in' or 'out',}

        wiz_id = self.wiz_add_materials_model.create(cr, uid, {})

        onchange_vals = self.material_line_model.onchange_product_id(
                cr, uid, [], product_id, context)

        vals = {'wiz_id': wiz_id, 'product_id': product_id,
                'location_id': location_id,
                'product_uom_qty': qty}

        vals.update(onchange_vals['value'])
        line0_id = self.material_line_model.create(cr, uid, vals, context)

        self.wiz_add_materials_model.confirm_materials(cr, uid, [wiz_id],
                context)
        
        wiz = self.wiz_add_materials_model.browse(cr, uid, wiz_id, context)
        return wiz

    def _transfer_picking(self, pick):
        cr, uid = self.cr, self.uid

        # Confirmamos el picking
        self.stock_picking.draft_force_assign(cr, uid, [pick.id], {'from_service': True})
        self.stock_picking.force_assign(cr, uid, [pick.id])
        
        context = {'active_id': pick.id,
                  'active_model': 'stock.picking',
                  'active_ids': [pick.id]}
        wiz_partial_id = self.partial_picking.create(cr, uid, {}, context)
        self.partial_picking.do_partial(cr, uid, [wiz_partial_id])

        pick.refresh()
        return pick

class TestSuitcase(common.TransactionCase):
    def setUp(self):
        super(TestSuitcase, self).setUp()
        cr, uid, context = self.cr, self.uid, {}

        # Datos a utilizar
        data_model = self.registry('ir.model.data')
        self.get_ref = partial(data_model.get_object_reference,
                               self.cr, self.uid)

        self.common_suitcase_func = CommonSuitcaseFunc(cr, uid, self.registry)

        self.product_product = self.registry('product.product')
        self.service_order_model = self.registry('service.order')
        self.suitcase_model = self.registry('stock.suitcase')
#        self.stock_location = self.registry('stock.location')
#        self.stock_move = self.registry('stock.move')
#        self.stock_partial_move = self.registry('stock.partial.move')
#        self.stock_partial_move_line = self.registry('stock.partial.move.line')
#        self.partial_picking_line = self.registry('stock.partial.picking.line')

        __, self.partner_id = self.get_ref('base', 'res_partner_7')
        __, self.type_maintenance_id = self.get_ref('service', 'service_type_maintenance')
        __, self.type_installation_id = self.get_ref('service', 'service_type_installation')
        __, self.system_1_id = self.get_ref('service', 'system_1')
        __, self.component_1_id = self.get_ref('service', 'component_1')
        __, self.employee1_id = self.get_ref('hr', 'employee_fp')
        __, self.location_components_id = self.get_ref('stock', 'stock_location_components')
        __, self.product1_id = self.get_ref('product', 'product_product_3')
        __, self.product2_id = self.get_ref('product', 'product_product_4')
        __, self.product3_id = self.get_ref('product', 'product_product_5')



    def test_01_suitcase_create(self):
        """Testeamos la creacion de una valija y su confirmacion."""
        cr, uid = self.cr, self.uid

        # Creamos la valija
        suitcase = self.common_suitcase_func._create_suitcase('Test Suitcase',
                self.employee1_id)

        # Chequeamos valores iniciales, no tiene que haber location
        self.assertTrue(suitcase)
        self.assertEquals(suitcase.state, 'draft')
        self.assertFalse(suitcase.location_id)

        # Luego de la confirmacion, chequeamos estado y location
        self.suitcase_model.confirm(cr, uid, [suitcase.id])
        suitcase.refresh()
        self.assertEquals(suitcase.state, 'available')
        self.assertTrue(suitcase.location_id)

    def test_05_suitcase_add_and_remove_materials(self):
        """Testeamos el llenado de una valija con materiales y luego le quitamos algunos."""
        cr, uid = self.cr, self.uid

        # Creamos la valija y la confirmamos para cargarle materiales
        suitcase = self.common_suitcase_func._create_suitcase('Test Suitcase',
                self.employee1_id)
        self.suitcase_model.confirm(cr, uid, [suitcase.id])

        # Chequemos que la valija no contenga materiales
        self.assertFalse(suitcase.current_materials_ids)
        self.assertFalse(suitcase.waiting_materials_ids)

        wiz = self.common_suitcase_func._add_materials_to_suitcase(
                suitcase, self.product1_id,
                self.location_components_id, 10.0)

        # Chequeamos que ahora haya materiales esperando en la valija
        suitcase.refresh()
        self.assertTrue(suitcase.waiting_materials_ids)
        self.assertFalse(suitcase.current_materials_ids)

        # El picking no tiene que estar realizado
        pick = wiz.picking_id
        self.assertEquals(pick.state, 'draft')

        self.common_suitcase_func._transfer_picking(pick)
        self.assertEquals(pick.state, 'done')

        # Chequeamos que ahora haya materiales en la valija
        # y no haya ninguno esperando
        suitcase.refresh()
        self.assertFalse(suitcase.waiting_materials_ids)
        self.assertTrue(suitcase.current_materials_ids)

        self.assertEquals(suitcase.current_materials_ids[0].product_qty, 10)


        # Ahora le quitamos materiales
        wiz = self.common_suitcase_func._rem_materials_to_suitcase(
                suitcase, self.product1_id,
                self.location_components_id, 3.0)

        # Chequeamos que no hayan materiales esperando
        # porque estamos quitandolos, pero si hay materiales actualmente
        # en la valija
        suitcase.refresh()
        self.assertFalse(suitcase.waiting_materials_ids)
        self.assertTrue(suitcase.current_materials_ids)
        self.assertEquals(suitcase.current_materials_ids[0].product_qty, 10.0)

        # El picking no tiene que estar realizado
        pick = wiz.picking_id
        self.assertEquals(pick.state, 'draft')

        self.common_suitcase_func._transfer_picking(pick)
        suitcase.refresh()
        self.assertEquals(pick.state, 'done')
        self.assertEquals(suitcase.current_materials_ids[0].product_qty, 7.0)
